@@ -21,7 +21,7 @@
  e-mail: russell@icrf.icnet.uk       e-mail gjb@bioch.ox.ac.uk
  WWW: http://bonsai.lif.icnet.uk/    WWW: http://geoff.biop.ox.ac.uk/
 
- The WORK is Copyright (1995) University of Oxford
+ The WORK is Copyright (1992,1993,1995,1996) University of Oxford
 	Administrative Offices
 	Wellington Square
 	Oxford OX1 2JD U.K.
@@ -34,21 +34,18 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <math.h>
-#include <malloc.h>
 #include <time.h>
 
-#include "general.h"
-#include "domain.h"
-#include "stamprel.h"
+#include <general.h>
+#include <domain.h>
+#include <stamprel.h>
 
 #define MAX_SEQ_LEN 2000
 #define PRECISION 1000
 
 /* avestruc: reads in a stamp output file and generates an average structure file */
-main(argc, argv)
-int argc; 
-char *argv[];
-{
+main(int argc, char *argv[]) {
+
 	int i,j,k,l,test;
 	int ndomain,total,add,nbloc;
 	int nstamp,nseq,nstamppos;
@@ -57,17 +54,36 @@ char *argv[];
 	int stampwindow;
 	int polyA,nrel;
 	int ncoords,nres;
-	int all_aligned;
+	int all_aligned,sidechain;
+	int ident,identical;
+	int cons,conserved;
+	int AApt;
+	int npolar, nhydrophobic, naromatic, nsmall, ntiny;
+        int npositive, nnegative, ncharged, naliphatic, nbranch;
+	int ignore;
+
 
 	int *pointer;
 	int *counter;
 	int *all;
-	int *rel;
+	int *rel,*realrel;
 	int **avecoords;
 	int ***n,***c,***o,***cb;
 	
 
 	char filename[200],outfile[200];
+	static char *AA3[]={
+           "ALA", "ASX", "CYS", "ASP", "GLU", /* 4 */
+           "PHE", "GLY", "HIS", "ILE", "UNK", /* 9 */
+           "LYS", "LEU", "MET", "ASN", "UNK", /* 14 */
+           "PRO", "GLN", "ARG", "SER", "THR", /* 19 */
+           "UNK", "VAL", "TRP", "UNK", "TYR", /* 24 */
+           "GLX", "SMA", "TIN", "POL", "HYD", /* 29  small, tiny, polar, hydrophobic */
+           "POS", "NEG", "CHA", "ARO", "ALI", /* 34  positive, negative, charged, aromatic, aliphatic */
+           "BRA"  			      /* 39  branched */
+        };
+
+
 	char stampchar;
 
 	char *env;
@@ -84,11 +100,12 @@ char *argv[];
 	struct seqdat *bloc;
 	struct stampdat *stamp;
 	struct brookn *numb;
+	struct side_chain **side; 
 
 	if(argc<3) exit_error();
 	
 	/* defaults */
-	polyA=0;
+	polyA=0; sidechain=0;
 	stampchar='G';
 	stampwindow=3;
 	stampthresh=6.0;
@@ -103,6 +120,9 @@ char *argv[];
 	average=(float*)malloc(3*sizeof(float));
 	buff=(char*)malloc(100*sizeof(char));
 	all_aligned=0;
+	ident=0;
+	cons=0;
+	ignore=0;
 
 	for(i=1; i<argc; ++i) {
 	   if(argv[i][0]!='-') exit_error();
@@ -124,11 +144,19 @@ char *argv[];
              i++;
 	   } else if(strcmp(&argv[i][1],"polyA")==0) {
 	     polyA=1;
+	   } else if(strcmp(&argv[i][1],"ident")==0) {
+             ident=1;
+	   } else if(strcmp(&argv[i][1],"cons")==0) {
+             cons=1;
+	   } else if(strcmp(&argv[i][1],"ignore")==0) {
+	     if((i+1)>=argc) exit_error();
+	     sscanf(argv[i+1],"%d",&ignore);
+	     i++;
 	   } else if(strcmp(&argv[i][1],"o")==0) {
 	     if((i+1)>=argc) exit_error();
              strcpy(&outfile[0],argv[i+1]);
              i++;
-	   } else if(strcmp(&argv[i][1],"aligned")==0) {
+	   } else if(strcmp(&argv[i][1],"aligned")==0 || strcmp(&argv[i][1],"alignment")==0) {
 	     all_aligned=1;
 	   } else {
 	     exit_error();
@@ -141,7 +169,7 @@ char *argv[];
       	}
 	/* read in coordinate locations and initial transformations */
 	if((TRANS = fopen(filename,"r")) == NULL) {
-	   fprintf(stderr,"error: file %s does not exist\n",argv[1]);
+	   fprintf(stderr,"error: file %s does not exist\n",filename);
 	   exit(-1);
 	}
 	if((OUT=fopen(outfile,"w"))==NULL) {
@@ -174,6 +202,11 @@ char *argv[];
 	  aa=(char*)malloc(MAX_SEQ_LEN*sizeof(char));
 	  numb=(struct brookn*)malloc(MAX_SEQ_LEN*sizeof(struct brookn));
 	}
+	if(sidechain) {
+	  side=(struct side_chain**)malloc(ndomain*sizeof(struct side_chain*));
+	}
+
+
 	for(i=0; i<ndomain; ++i) {
 	  if(polyA) {
 	    n[i]=(int**)malloc(MAX_SEQ_LEN*sizeof(int*));
@@ -181,6 +214,10 @@ char *argv[];
 	    c[i]=(int**)malloc(MAX_SEQ_LEN*sizeof(int*));
 	    cb[i]=(int**)malloc(MAX_SEQ_LEN*sizeof(int*));
 	  }
+	  if(sidechain) {
+	     side[i]=(struct side_chain*)malloc(MAX_SEQ_LEN*sizeof(struct side_chain));
+	  }
+
 	  for(j=0; j<domain[i].nobj; ++j) {
 	     if(domain[i].start[j].cid=='_') domain[i].start[j].cid=' ';
 	     if( domain[i].start[j].in=='_') domain[i].start[j].in=' ';
@@ -195,7 +232,10 @@ char *argv[];
 	rewind(TRANS);
 	bloc=(struct seqdat*)malloc((ndomain*2+2)*sizeof(struct seqdat));
 	printf(" ");
-	Agetbloc(TRANS,bloc,&nbloc);
+	if(Agetbloc(TRANS,bloc,&nbloc)==-1) {
+           fprintf(stderr,"error: alignment file %s appears to be incomplete %s\n",filename);                     
+            exit(-1); 
+        }
 	bloclen=strlen(&bloc[1].seq[1]);
 	if(nbloc!=(ndomain*2+1)) {
 	   fprintf(stderr,"error: number of domains and alignment disagree\n");
@@ -207,8 +247,18 @@ char *argv[];
 	fprintf(OUT,"REMARK   \n");
 	/* read in the STAMP output and get reliable regions */
 	rewind(TRANS);
+	stamp=(struct stampdat*)malloc(100*sizeof(struct stampdat));
+	if(getstampdat(stamp,TRANS,&nstamp,&nseq,&nstamppos,MAX_SEQ_LEN)==-1) exit(-1);
+	if((rel=getstamprel(stamp,nstamp,bloclen,stampchar,stampthresh,stampwindow))==NULL) exit(-1);
+	realrel=(int*)malloc(bloclen*sizeof(int));
+
+	nrel=0;
+        for(i=0; i<bloclen; ++i) {
+             if(rel[i]==1) nrel++;
+	     realrel[i]=rel[i];
+	}
+
 	if(all_aligned) {
-	   rel=(int*)malloc(MAX_SEQ_LEN*sizeof(int));
 	   for(i=0; i<bloclen; ++i) {
 	     k=0;
 	     for(j=0; j<ndomain; ++j) {
@@ -217,18 +267,16 @@ char *argv[];
 	     if(k==ndomain) rel[i]=1;
 	     else rel[i]=0;
 	   }
-        } else { 
-	   stamp=(struct stampdat*)malloc(100*sizeof(struct stampdat));
-	   if(getstampdat(stamp,TRANS,&nstamp,&nseq,&nstamppos,MAX_SEQ_LEN)==-1) exit(-1);
-	   if((rel=getstamprel(stamp,nstamp,bloclen,stampchar,stampthresh,stampwindow))==NULL) exit(-1);
-	}
-	nrel=0;
-	   for(i=0; i<bloclen; ++i) {
-             if(rel[i]==1) nrel++;
 	}
 	fprintf(OUT,"REMARK %4d STAMP reliable or core positions found\n",nrel);
+	fprintf(OUT,"REMARK  (STAMP parms '%c' %5.2f %2d) \n",stampchar,stampthresh,stampwindow);
 	fprintf(OUT,"REMARK The characters to the far right of the coordinates show\n");
 	fprintf(OUT,"REMARK  the residues occuring at each position within the alignment\n");
+	if(all_aligned) {
+	  fprintf(OUT,"REMARK  All aligned positions were used to generate the\n");
+	  fprintf(OUT,"REMARK   coordinates below\n");
+	}
+	
 	fclose(TRANS);
 
 	printf(" Reading coordinates...\n");
@@ -248,7 +296,10 @@ char *argv[];
 	   for(j=0; j<domain[i].nobj; ++j) {
 	       if(igetca(PDB,&domain[i].coords[total],&domain[i].aa[total],&domain[i].numb[total],
 		  &add,domain[i].start[j],domain[i].end[j],
-		  domain[i].type[j],(MAX_SEQ_LEN-total),domain[i].reverse[j],PRECISION,stdout)==-1) exit(-1);
+		  domain[i].type[j],(MAX_SEQ_LEN-total),domain[i].reverse[j],PRECISION,stdout)==-1) {
+		    fprintf(stderr,"Error in domain %s object %d \n",domain[i].id,j+1);
+		    exit(-1);
+	       }
 	       if(polyA) {
 		  rewind(PDB);
 		  if(igetgen(PDB,&n[i][total],&aa[total],&numb[total],&test,domain[i].start[j],domain[i].end[j],domain[i].type[j],N,(MAX_SEQ_LEN-total),domain[i].reverse[j],PRECISION,stdout)==-1) exit(-1);
@@ -278,6 +329,13 @@ char *argv[];
                         exit(-1);
                   }
 	       }
+	       if(sidechain) {  
+		  /* read all side chains into this sort of array of structures */
+		  if(igetside(PDB,side[i],&aa[total],&numb[total],&test,domain[i].start[j],domain[i].end[j],
+                    domain[i].type[j],(MAX_SEQ_LEN-total),domain[i].reverse[j],PRECISION,stdout)==-1) exit(-1);
+		}
+
+
 	       switch(domain[i].type[j]) {
 	 	  case 1: printf(" all residues"); break; 
 		  case 2: printf(" chain %c",domain[i].start[j].cid); break;
@@ -331,6 +389,51 @@ char *argv[];
 	ncoords=0; nres=0;
 	for(i=0; i<bloclen; ++i) {
 	  if(rel[i]==1) {
+	    AApt = 23; /* UNK by default */
+	    if(cons || ident) { /* Work out if the residue is conserved/identical */
+	        identical=1; 
+		for(j=0; j<ndomain; ++j) {
+		   if(bloc[j+1].seq[i+1]!=bloc[1].seq[i+1]) identical=0;
+	        }
+		if(identical) {
+		     AApt = (int)(bloc[1].seq[i+1]-'A');
+		     if(AApt<0 || AApt>26) { AApt = 23; } /* UNK if gone awry */
+		} else {
+		     AApt = 23; /* UNK if unconserved */
+		}
+	    }
+	    if(cons && AApt == 23) { /* If not identical, look for property conservation */
+		npolar=0; nhydrophobic=0; naromatic=0; nsmall=0; ntiny=0; 
+		npositive=0; nnegative=0; ncharged=0; naliphatic=0; nbranch=0;
+		for(j=0; j<ndomain; ++j) {
+		    switch(bloc[j+1].seq[i+1]) {
+			case 'R': case 'K': { npositive++; ncharged++; npolar++; } break;
+			case 'H': { npositive++; ncharged++; npolar++; naromatic++; } break;
+		        case 'E': case 'D': { nnegative++; ncharged++; npolar++; } break;
+			case 'Q': case 'N': { npolar++; } break;
+			case 'T': { nsmall++; nbranch++; npolar++; } break;
+			case 'S': case 'P': case 'G': { nsmall++; ntiny++; npolar++; } break;
+			case 'A': { nsmall++; ntiny++; naliphatic++; } break;
+			case 'V': { nsmall++; naliphatic++; nbranch++; nhydrophobic++; } break;
+			case 'I': { naliphatic++; nbranch++; nhydrophobic++; } break;
+			case 'L': case 'M': { naliphatic++; nhydrophobic++; } break;
+			case 'C': { nsmall++; ntiny++; nhydrophobic++; } break;
+			case 'F': case 'W': { nhydrophobic++; naromatic++; } break;
+			case 'Y': { nhydrophobic++; naromatic++;  npolar++; } break;
+		    }
+		 }
+		 if(npositive>=(ndomain-ignore)) { AApt= 30; }
+		 else if(nnegative>=(ndomain-ignore)) { AApt = 31; }
+		 else if(ncharged>=(ndomain-ignore)) { AApt = 32; }
+		 else if(npolar>=(ndomain-ignore)) { AApt = 28; }
+		 else if(naromatic>=(ndomain-ignore)) { AApt = 33; }
+		 else if(ntiny>=(ndomain-ignore)) { AApt = 27; }
+		 else if(nbranch>=(ndomain-ignore)) { AApt = 35; }
+		 else if(nsmall>=(ndomain-ignore)) { AApt = 26; }
+		 else if(naliphatic>=(ndomain-ignore)) { AApt = 34; }
+		 else if(nhydrophobic>=(ndomain-ignore)) { AApt = 29; }
+		 else { AApt = 23; } 
+	    }
 	    for(j=0; j<3; ++j) average[j]=0.0;
 	    if(polyA) {
 		/* N atoms */
@@ -338,9 +441,11 @@ char *argv[];
 		for(j=0; j<ndomain; ++j) {
                    for(k=0; k<3; ++k) average[k]+=((float)n[j][pointer[j]][k]/(float)ndomain)/(float)PRECISION;
                 }
-		fprintf(OUT,"ATOM  %5d  N   ALA Z%4d    %8.3f%8.3f%8.3f  0.00  0.00   0 \n",
-                  ncoords+1,nres+1,average[0],average[1],average[2]);
+		fprintf(OUT,"ATOM  %5d  N   %3s Z%4d    %8.3f%8.3f%8.3f  0.00 ",
+                  ncoords+1,AA3[AApt],nres+1,average[0],average[1],average[2]);
 		ncoords++;
+		if(realrel[i]==1) fprintf(OUT," 1.00   0 \n");
+		else fprintf(OUT,"99.00   0 \n");
 	    }
 	    /* CA atoms */	
  	    for(j=0; j<3; ++j) average[j]=0.0;
@@ -348,9 +453,11 @@ char *argv[];
 		for(k=0; k<3; ++k) average[k]+=((float)domain[j].coords[pointer[j]][k]/(float)ndomain)/(float)PRECISION;
 	    }
 	    /* Output the average coordiates */ 
-	    sprintf(buff,"ATOM  %5d  CA  ALA Z%4d    %8.3f%8.3f%8.3f  0.00  0.00   0 ",
-		ncoords+1,nres+1,average[0],average[1],average[2]);
-	    fprintf(OUT,"%s",buff);
+	    fprintf(OUT,"ATOM  %5d  CA  %3s Z%4d    %8.3f%8.3f%8.3f  0.00 ",
+		ncoords+1,AA3[AApt],nres+1,average[0],average[1],average[2]);
+	    if(realrel[i]==1) fprintf(OUT," 1.00   0 ");
+            else fprintf(OUT,"99.00   0 ");
+
 	    for(j=0; j<ndomain; ++j) 
                if(j<13) fprintf(OUT,"%c",bloc[j+1].seq[i+1]); 
 	    fprintf(OUT,"\n");
@@ -361,22 +468,29 @@ char *argv[];
                 for(j=0; j<ndomain; ++j) {
                    for(k=0; k<3; ++k) average[k]+=((float)c[j][pointer[j]][k]/(float)ndomain)/(float)PRECISION;
                 }
-                fprintf(OUT,"ATOM  %5d  C   ALA Z%4d    %8.3f%8.3f%8.3f  0.00  0.00   0 \n",
-                  ncoords+1,nres+1,average[0],average[1],average[2]);
+                fprintf(OUT,"ATOM  %5d  C   %3s Z%4d    %8.3f%8.3f%8.3f  0.00 ",
+                  ncoords+1,AA3[AApt],nres+1,average[0],average[1],average[2]);
                 ncoords++;
+		if(realrel[i]==1) fprintf(OUT," 1.00   0 \n");
+                else fprintf(OUT,"99.00   0 \n");
+
                 for(j=0; j<3; ++j) average[j]=0.0;
                 for(j=0; j<ndomain; ++j) {
                    for(k=0; k<3; ++k) average[k]+=((float)o[j][pointer[j]][k]/(float)ndomain)/(float)PRECISION;
                 }
-                fprintf(OUT,"ATOM  %5d  O   ALA Z%4d    %8.3f%8.3f%8.3f  0.00  0.00   0 \n",
-                  ncoords+1,nres+1,average[0],average[1],average[2]);
+                fprintf(OUT,"ATOM  %5d  O   %3s Z%4d    %8.3f%8.3f%8.3f  0.00 ",
+                  ncoords+1,AA3[AApt],nres+1,average[0],average[1],average[2]);
+	        if(realrel[i]==1) fprintf(OUT," 1.00   0 \n");
+                else fprintf(OUT,"99.00   0 \n");
                 ncoords++;
                 for(j=0; j<3; ++j) average[j]=0.0;
                 for(j=0; j<ndomain; ++j) {
                    for(k=0; k<3; ++k) average[k]+=((float)cb[j][pointer[j]][k]/(float)ndomain)/(float)PRECISION;
                 }
-                fprintf(OUT,"ATOM  %5d  CB  ALA Z%4d    %8.3f%8.3f%8.3f  0.00  0.00   0 \n",
-                  ncoords+1,nres+1,average[0],average[1],average[2]);
+                fprintf(OUT,"ATOM  %5d  CB  %3s Z%4d    %8.3f%8.3f%8.3f  0.00 ",
+                  ncoords+1,AA3[AApt],nres+1,average[0],average[1],average[2]);
+	        if(realrel[i]==1) fprintf(OUT," 1.00   0 \n");
+                else fprintf(OUT,"99.00   0 \n");
                 ncoords++;
 	    }
 	    nres++; 
@@ -394,5 +508,6 @@ int exit_error()
 	 fprintf(stderr,"format:  avestruc -f <file> -c <char> -w <win> -t <thresh> -polyA -aligned\n");
 	 fprintf(stderr,"          where -polyA ==> poly alanine structure (default is CAs only)\n");
  	 fprintf(stderr,"                -aligned ==> consider all aligned positions\n");
+	 fprintf(stderr,"                -cons ==> label conserved AA positions in the file\n");
 	 exit(-1);
 }
